@@ -21,19 +21,22 @@
 
 from __future__ import absolute_import, division, print_function, unicode_literals
 
+import logging
 import math
 import pprint
 import signal
+import sys
+import time
+import resource
 
 from collections import OrderedDict
-from datetime import datetime
-from threading import Event
+from threading import Event, Thread
+from multiprocessing import Pool, Manager, cpu_count
 
-from flent import runners
-from flent.util import classname
-from flent.loggers import get_logger
+from flent import runners, loggers
+from flent.util import classname, utcfromtimestamp
 
-logger = get_logger(__name__)
+logger = loggers.get_logger(__name__)
 
 
 def new(settings):
@@ -50,12 +53,8 @@ def new(settings):
         raise RuntimeError("Error loading %s: %s." % (cname, e))
 
 
-class GracefulShutdown(Exception):
-    pass
-
-
 def handle_usr1(signal, frame):
-    raise GracefulShutdown()
+    raise KeyboardInterrupt()
 
 
 class Aggregator(object):
@@ -71,6 +70,15 @@ class Aggregator(object):
 
         self.postprocessors = []
 
+    def read_log_queue(self, queue):
+        while True:
+            try:
+                msg = queue.get()
+                logging.getLogger().handle(msg)
+                queue.task_done()
+            except (EOFError, OSError):
+                return
+
     def add_instance(self, name, config):
         instance = dict(config)
 
@@ -100,13 +108,65 @@ class Aggregator(object):
 
         self.instances[name] = instance
 
+    def check_rlimit(self):
+        ni = len(self.instances)
+        min_f = ni * 4 + 4
+        min_p = ni + cpu_count()
+
+        try:
+            nf, nf_h = resource.getrlimit(resource.RLIMIT_NOFILE)
+            np, np_h = resource.getrlimit(resource.RLIMIT_NPROC)
+        except OSError as e:
+            logger.debug("Couldn't get rlimit (%s) - may be harmless, so ignoring", e)
+
+        if nf != resource.RLIM_INFINITY and nf < min_f:
+            logger.debug("Raising RLIMIT_NOFILE from %d to required minimum %d",
+                         nf, min_f)
+            try:
+                resource.setrlimit(resource.RLIMIT_NOFILE, (min_f, nf_h))
+            except OSError as e:
+                raise RuntimeError(f"Couldn't raise RLIMIT_NOFILE to required "
+                                   f"{min_f}: {e} - try manually adjusting `ulimit -n`")
+
+        if np != resource.RLIM_INFINITY and np < min_p:
+            logger.debug("Raising RLIMIT_NPROC from %d to required minimum %d",
+                         np, min_p)
+            try:
+                resource.setrlimit(resource.RLIMIT_NPROC, (min_p, np_h))
+            except OSError:
+                raise RuntimeError(f"Couldn't raise RLIMIT_NPROC to required "
+                                   f"{min_p}: {e} - try manually adjusting `ulimit -u`")
+
     def aggregate(self, results):
         raise NotImplementedError()
 
+    def parse_results_parallel(self):
+        with Manager() as mngr:
+            queue = mngr.Queue(10)
+            with Pool(initializer=loggers.set_queue_handler, initargs=(queue, )) as parser_pool:
+                q_thread = Thread(target=self.read_log_queue, args=(queue, ), daemon=True)
+                q_thread.start()
+
+                res = []
+                for t in self.threads.values():
+                    res.extend(t.do_parse(parser_pool))
+
+                for r in res:
+                    r.wait()
+
+                queue.join()
+                parser_pool.close()
+                parser_pool.join()
+
+    def parse_results_direct(self):
+        for t in self.threads.values():
+            t.do_parse(None)
+
     def collect(self):
         """Create a ProcessRunner thread for each instance and start them. Wait
         for the threads to exit, then collect the results."""
 
+        self.check_rlimit()
         signal.signal(signal.SIGUSR1, handle_usr1)
 
         result = {}
@@ -127,26 +187,47 @@ class Aggregator(object):
                 self.threads[n] = t
 
             # Start in a separate loop once we're sure we successfully created
-            # all runners
+            # all runners. To prevent fork() from inheriting all the management
+            # threads, run a separate loop to fork off all threads before
+            # starting the actual supervisor threads afterwards
+            num_forks = 0
+            test_start = time.monotonic()
             for t in self.threads.values():
-                t.start()
+                if hasattr(t, "fork"):
+                    num_forks += t.fork()
+
+            fork_end = time.monotonic()
+            logger.debug("Forked %d processes in %f seconds", num_forks, fork_end - test_start)
+
+            num_threads = 0
+            for t in self.threads.values():
+                num_threads += t.start()
+
+            threads_end = time.monotonic()
+            logger.debug("Started %d threads in %f seconds", num_threads, threads_end - fork_end)
 
-            shutting_down = False
             for n, t in list(self.threads.items()):
-                while t.is_alive():
-                    try:
-                        t.join(1)
-                    except GracefulShutdown:
-                        if not shutting_down:
-                            logger.info(
-                                "SIGUSR1 received; initiating graceful shutdown. "
-                                "This may take a while...")
-                            self.kill_runners(graceful=True)
-                            shutting_down = True
-                        else:
-                            logger.info(
-                                "Already initiated graceful shutdown. "
-                                "Patience, please...")
+                t.join()
+
+            run_end = time.monotonic()
+            logger.debug("Ran test in %f seconds", run_end - threads_end)
+
+            if sys.platform in ('darwin', 'win32'):
+                logger.debug("Parsing results in main thread on %s platform", sys.platform)
+                self.parse_results_direct()
+            else:
+                logger.debug("Parsing results in parallel on %s platform", sys.platform)
+                self.parse_results_parallel()
+
+            for t in self.threads.values():
+                # used by SsRunner to copy over results from duplicate runners
+                # after parsing has completed
+                t.post_parse()
+
+            parse_end = time.monotonic()
+            logger.debug("Parsing completed in %f seconds", parse_end - run_end)
+
+            for n, t in self.threads.items():
 
                 metadata['series'][n] = t.metadata
                 if t.test_parameters:
@@ -178,8 +259,20 @@ class Aggregator(object):
                                 "Duplicate key '%s' from child runner" % key)
                         result[key] = v
 
-        except KeyboardInterrupt:
-            logger.debug("Received SIGINT")
+            compute_end = time.monotonic()
+            logger.debug("Finished post-test computation in %f seconds", compute_end - parse_end)
+
+            for t in self.threads.values():
+                t.close()
+
+            test_end = time.monotonic()
+            logger.debug("Ran full test in %f seconds", test_end - test_start)
+
+            # logger cache may retain references to runners, so flush it to make
+            # sure they all get freed up
+            loggers.flush_cache()
+
+        except BaseException:
             self.kill_runners()
             raise
 
@@ -189,10 +282,16 @@ class Aggregator(object):
         signal.signal(signal.SIGUSR1, signal.SIG_DFL)
         return result, metadata, raw_values
 
-    def kill_runners(self, graceful=False):
+    def kill_runners(self):
+        if self.killed:
+            return
+
         self.killed = True
-        for t in list(self.threads.values()):
-            t.kill(graceful)
+        for t in self.threads.values():
+            t.kill()
+        for t in self.threads.values():
+            t.join()
+            t.close()
 
     def postprocess(self, result):
         logger.debug("Postprocessing data using %d postprocessors", len(self.postprocessors))
@@ -271,7 +370,7 @@ class TimeseriesAggregator(Aggregator):
         if steps > 10**6:
             raise RuntimeError("Refusing to iterate more than 1 million steps during aggregation (got %d)" % steps)
 
-        results.meta('T0', datetime.utcfromtimestamp(t_0))
+        results.meta('T0', utcfromtimestamp(t_0))
 
         for s in range(steps):
             time_label = self.step * s
