@@ -25,6 +25,7 @@ import io
 import math
 import os
 import re
+import sys
 import warnings
 
 from flent import combiners
@@ -38,7 +39,6 @@ from argparse import SUPPRESS
 from functools import reduce
 from itertools import cycle, islice, chain
 from collections import OrderedDict
-from distutils.version import LooseVersion
 
 logger = get_logger(__name__)
 
@@ -46,15 +46,14 @@ try:
     import matplotlib
     import numpy as np
     HAS_MATPLOTLIB = True
-    MPL_VER = LooseVersion(matplotlib.__version__)
-    if MPL_VER < LooseVersion("1.5"):
+    mpl_maj, _ = matplotlib.__version__.split(".", 1)
+    if mpl_maj in ('1', '2'):
         logger.warning("Cannot use old matplotlib version %s, please upgrade!",
                        matplotlib.__version__)
         raise ImportError("Matplotlib %s too old" % matplotlib.__version__)
 except ImportError as e:
     logger.debug("Unable to import matplotlib: %s", e)
     HAS_MATPLOTLIB = False
-    MPL_VER = LooseVersion("0")
 
 PLOT_KWARGS = (
     'alpha',
@@ -151,7 +150,7 @@ def init_matplotlib(output, use_markers, load_rc):
             raise RuntimeError(
                 "Unrecognised file format for output '%s'" % output)
 
-    elif not os.getenv("DISPLAY"):
+    elif (sys.platform == 'linux' and not os.getenv("DISPLAY")):
         matplotlib.use("agg")
 
     from matplotlib import pyplot
@@ -440,9 +439,9 @@ def add_plotting_args(parser):
     parser.add_argument(
         "--filter-regexp",
         action="append", type=str, dest="FILTER_REGEXP", metavar="REGEXP",
-        default=[], help="Filter labels (regex). Filter out supplied regular "
-        "expression from label names. Can be specified multiple times, in which "
-        "case the regular expressions will be filtered in the order specified.")
+        default=[], help="Filter labels (regex). Filter out (remove) text matching "
+        "the supplied regular expression from label names. Can be specified multiple "
+        "times, in which case the filters will be applied in the order specified.")
 
     parser.add_argument(
         "--override-label",
@@ -655,7 +654,7 @@ class Plotter(ArgParam):
             self.figure._original_dpi = self.figure.dpi
 
     def __del__(self):
-        if not self.disable_cleanup:
+        if not getattr(self, "disable_cleanup", False):
             try:
                 pyplot.close(self.figure)
             except Exception:
@@ -663,6 +662,10 @@ class Plotter(ArgParam):
 
     def __getstate__(self):
         state = self.__dict__.copy()
+        if 'figure' in state:
+            fig = state['figure']
+            if getattr(fig, "canvas", None) and getattr(fig.canvas, "manager", None):
+                fig.canvas.manager = None
         return state
 
     def init(self, config=None, axis=None):
@@ -947,6 +950,24 @@ class Plotter(ArgParam):
         finally:
             pdf.close()
 
+    def _get_renderer(self):
+
+        # Matplotlib 3.6 reorganised the renderer caching logic and introduced
+        # the _get_renderer() method of the figure class, so try that first
+        try:
+            return self.figure._get_renderer()
+        except AttributeError:
+            pass
+
+        # Older versions of matplotlib cached the renderer in the figure itself,
+        # but it may not exist until after figure is drawn
+        renderer = getattr(self.figure, '_cachedRenderer', None)
+        if not renderer:
+            self.figure.canvas.draw()
+            renderer = getattr(self.figure, '_cachedRenderer', None)
+
+        return renderer
+
     def build_tight_layout(self, artists):
         args = None
         if self.fallback_layout:
@@ -954,7 +975,7 @@ class Plotter(ArgParam):
         try:
             self.figure.savefig(io.BytesIO())
 
-            renderer = self.figure._cachedRenderer
+            renderer = self._get_renderer()
             right = x_max = self.figure.get_figwidth() * self.figure.dpi
             top = y_max = self.figure.get_figheight() * self.figure.dpi
             vsp = 0.02 * self.figure.dpi
@@ -1033,15 +1054,9 @@ class Plotter(ArgParam):
            and not self.legend_placement \
            and self.legends:
 
-            # Make sure we have a renderer to get size from
-            renderer = getattr(self.figure, '_cachedRenderer', None)
-            if not renderer:
-                self.figure.canvas.draw()
-                renderer = getattr(self.figure, '_cachedRenderer', None)
-
             try:
                 legend_width = max(
-                    [l.get_window_extent(renderer).width for l in self.legends])
+                    [l.get_window_extent(self._get_renderer()).width for l in self.legends])
             except Exception as e:
                 logger.debug("Error getting legend sizes: %s", e)
                 return
@@ -1223,7 +1238,10 @@ class Plotter(ArgParam):
             axis.set_ylim(0, top_scale)
         else:
             if self.log_base:
-                axis.set_yscale('log', basey=self.log_base)
+                try:
+                    axis.set_yscale('log', base=self.log_base)
+                except TypeError:
+                    axis.set_yscale('log', basey=self.log_base)
                 axis.set_ylim(max(0, btm_scale), top_scale)
             else:
                 axis.set_ylim(btm_scale, top_scale)
@@ -1869,7 +1887,10 @@ class CdfPlotter(Plotter):
             axis.set_xlim(left=min(min_value, axis.get_xlim()[0]))
 
         if self.log_base:
-            axis.set_xscale('log', basex=self.log_base)
+            try:
+                axis.set_xscale('log', base=self.log_base)
+            except TypeError:
+                axis.set_xscale('log', basex=self.log_base)
 
         for a, b in zip(config['axes'], self.bounds_x):
             a.set_xbound(b)
