@@ -24,6 +24,8 @@ from __future__ import absolute_import, division, print_function, unicode_litera
 import math
 import os
 import re
+import gzip
+import bz2
 
 from datetime import datetime
 from calendar import timegm
@@ -33,7 +35,7 @@ from collections import OrderedDict
 
 from flent import transformers
 from flent.loggers import get_logger
-from flent.util import gzip_open, bz2_open, parse_date, format_date
+from flent.util import parse_date, format_date, utcnow
 
 try:
     import ujson as json
@@ -77,7 +79,7 @@ MAX_FILENAME_LEN = 250  # most filesystems have 255 as their limit
 
 # Time settings will be serialised as ISO timestamps and stored in memory as
 # datetime instances
-TIME_SETTINGS = ("TIME", "BATCH_TIME", "T0")
+TIME_SETTINGS = ("TIME", "T0", "BATCH_TIME")
 
 _EMPTY = object()
 
@@ -139,9 +141,9 @@ class ResultSet(object):
         self._raw_keys = None
         self.metadata = SeparatorDict(kwargs, sep=":")
         self.SUFFIX = SUFFIX
-        self.t0 = None
+        self._t0 = None
         if 'TIME' not in self.metadata or self.metadata['TIME'] is None:
-            self.metadata['TIME'] = datetime.utcnow()
+            self.metadata['TIME'] = utcnow()
         if 'NAME' not in self.metadata or self.metadata['NAME'] is None:
             raise RuntimeError("Missing name for resultset")
         if 'DATA_FILENAME' not in self.metadata \
@@ -163,6 +165,9 @@ class ResultSet(object):
                                                   fmt="%Y-%m-%d %H:%M:%S"))
             self.long_title = self.title
 
+        if 'SERIES_META' not in self.metadata:
+            self.metadata['SERIES_META'] = {}
+
     def meta(self, key=None, value=_EMPTY):
         if key:
             if value is not _EMPTY:
@@ -170,6 +175,13 @@ class ResultSet(object):
             return self.metadata[key]
         return self.metadata
 
+    def series_meta(self, series, key=None, value=_EMPTY):
+        if key:
+            if value is not _EMPTY:
+                self.metadata['SERIES_META'][series][key] = value
+            return self.metadata['SERIES_META'][series][key]
+        return self.metadata['SERIES_META'][series]
+
     def label(self):
         return self._label or self.metadata["TITLE"] \
             or format_date(self.metadata["TIME"])
@@ -204,6 +216,7 @@ class ResultSet(object):
     def create_series(self, series_names):
         for n in series_names:
             self._results[n] = []
+            self.metadata['SERIES_META'][n] = {}
 
     def append_datapoint(self, x, data):
         """Append a datapoint to each series. Missing data results in append
@@ -246,16 +259,25 @@ class ResultSet(object):
 
     def series(self, name, smooth=None):
         if name not in self._results:
-            logger.warning("Missing data points for series '%s'", name)
+            logger.debug("Missing data points for series '%s'", name)
             return [None] * len(self.x_values)
         if smooth:
             return self.smoothed(self._results[name], smooth)
         return self._results[name]
 
     def _calculate_t0(self):
-        self.t0 = timegm(self.metadata['T0'].timetuple(
+        self._t0 = timegm(self.metadata['T0'].timetuple(
         )) + self.metadata['T0'].microsecond / 1000000.0
 
+    def get_t0(self):
+        if self._t0 is None:
+            self._calculate_t0()
+        return self._t0
+
+    def set_t0(self, value):
+        self._t0 = value
+    t0 = property(get_t0, set_t0)
+
     def raw_series(self, name, absolute=False, raw_key=None):
         if name not in self.raw_values:
             raise KeyError(name)
@@ -263,9 +285,6 @@ class ResultSet(object):
         if raw_key is None:
             raw_key = 'val'
 
-        if self.t0 is None:
-            self._calculate_t0()
-
         for i in self.raw_values[name]:
             try:
                 x = i['t'] if absolute else i['t'] - self.t0
@@ -334,7 +353,7 @@ class ResultSet(object):
         return self.zipped()
 
     def __len__(self):
-        return len(self._x_values)
+        return len(self.raw_values)
 
     def __hash__(self):
         if self._loaded_from is None:
@@ -368,17 +387,14 @@ class ResultSet(object):
 
     def dump(self, fp):
         data = self.dumps()
-        if hasattr(data, "decode"):
-            data = data.decode()
-
         return fp.write(data)
 
     def dump_file(self, filename):
         try:
             if filename.endswith(".gz"):
-                o = gzip_open
+                o = gzip.open
             elif filename.endswith(".bz2"):
-                o = bz2_open
+                o = bz2.open
             else:
                 o = open
             with o(filename, "wt") as fp:
@@ -415,11 +431,12 @@ class ResultSet(object):
 
     def dump_dir(self, dirname):
         self._dump_file = os.path.join(dirname, self._gen_filename())
+        logger.debug("Writing data file to '%s'", self._dump_file)
         try:
             if self._dump_file.endswith(".gz"):
-                o = gzip_open
+                o = gzip.open
             elif self._dump_file.endswith(".bz2"):
-                o = bz2_open
+                o = bz2.open
             else:
                 o = open
             with o(self._dump_file, "wt") as fp:
@@ -450,9 +467,17 @@ class ResultSet(object):
         if 'TOTAL_LENGTH' not in metadata or metadata['TOTAL_LENGTH'] is None:
             metadata['TOTAL_LENGTH'] = max(obj['x_values'])
 
+        # We need the minimum timestamp to guess a timezone offset, which we
+        # store for subsequent values because it shouldn't be used for
+        # BATCH_TIME
+        min_t = 10**10
+        offset = None
+        for v in obj['raw_values'].values():
+            min_t = min([min_t] + [i['t'] for i in v if 't' in i])
         for t in TIME_SETTINGS:
             if t in metadata and metadata[t] is not None:
-                metadata[t] = parse_date(metadata[t])
+                metadata[t], offset = parse_date(metadata[t],
+                                                 min_t=min_t, offset=offset)
         rset = cls(SUFFIX=SUFFIX, **metadata)
         if absolute:
             t0 = metadata.get('T0', metadata.get('TIME'))
@@ -483,7 +508,7 @@ class ResultSet(object):
                 # relying on their presence in future code.
                 logger.debug("No raw values found; synthesising from parsed data")
 
-                t0 = parse_date(obj['metadata'].get(
+                t0, offset = parse_date(obj['metadata'].get(
                     'T0', obj['metadata'].get('TIME')))
                 x0 = timegm(t0.timetuple()) + t0.microsecond / 1000000.0
                 for name in obj['results'].keys():
@@ -602,9 +627,9 @@ class ResultSet(object):
         logger.debug("Loading data file %s", filename)
         try:
             if filename.endswith(".gz"):
-                o = gzip_open
+                o = gzip.open
             elif filename.endswith(".bz2") or filename.endswith(".flnt"):
-                o = bz2_open
+                o = bz2.open
             else:
                 o = open
             fp = o(filename, 'rt')
