@@ -38,8 +38,9 @@ except ImportError:
 from flent.build_info import VERSION
 from flent.testenv import TestEnvironment, TEST_PATH
 from flent.loggers import get_logger
-from flent.util import FuncAction, Update, keyval, keyval_int, ArgParser, \
-    token_split
+from flent.util import FuncAction, Update, AddHost, append_host, keyval, \
+    keyval_int, keyval_transformer, ArgParser, token_split, global_cache, \
+    utcnow
 from flent.plotters import add_plotting_args
 from flent import loggers, util, resultset, runners
 
@@ -59,7 +60,7 @@ DEFAULT_SETTINGS = {
     'LOCAL_HOST': socket.gethostname(),
     'DESCRIPTION': 'No description',
     'PLOTS': {},
-    'TIME': datetime.utcnow(),
+    'TIME': utcnow(),
     'BATCH_NAME': None,
     'BATCH_UUID': None,
 }
@@ -67,6 +68,10 @@ DEFAULT_SETTINGS = {
 DICT_SETTINGS = ('DATA_SETS', 'PLOTS')
 
 
+class _LOG_DEFER:
+    pass
+
+
 class Version(FuncAction):
 
     def __call__(*args):
@@ -80,14 +85,11 @@ class Version(FuncAction):
         except ImportError:
             logger.info("No matplotlib found. Plots won't be available.")
         try:
-            from PyQt5 import QtCore
-            logger.info("Using PyQt5 version %s.", QtCore.PYQT_VERSION_STR)
+            import qtpy
+            from qtpy import QtCore
+            logger.info("Using Qt: %s v%s.", qtpy.API, QtCore.__version__)
         except ImportError:
-            try:
-                from PyQt4 import QtCore
-                logger.info("Using PyQt4 version %s.", QtCore.PYQT_VERSION_STR)
-            except ImportError:
-                logger.info("No usable PyQt found. GUI won't work.")
+            logger.info("No usable Qt version found. GUI won't work.")
         sys.exit(0)
 
 
@@ -108,8 +110,11 @@ class LogFile(argparse.Action):
         super(LogFile, self).__init__(option_strings, dest, **kwargs)
 
     def __call__(self, parser, namespace, values, option_string=None):
-        loggers.setup_logfile(values)
-        setattr(namespace, self.dest, values)
+        if values is None:
+            setattr(namespace, self.dest, _LOG_DEFER)
+        else:
+            loggers.setup_logfile(values)
+            setattr(namespace, self.dest, values)
 
 
 class Debug(argparse.Action):
@@ -127,6 +132,7 @@ class ListTests(FuncAction):
 
     @classmethod
     def get_tests(cls, settings):
+        settings = settings.copy()
         tests = []
         for t in sorted([os.path.splitext(i)[0] for i in os.listdir(TEST_PATH)
                          if i.endswith('.conf')]):
@@ -274,6 +280,13 @@ parser.add_argument(
     action="store_false", dest="BATCH_SHUFFLE",
     help="Do not randomise the order of test runs within each batch.")
 
+parser.add_argument(
+    "--batch-no-timestamp",
+    action="store_false", dest="BATCH_TIMESTAMP",
+    help="Do not add a timestamp to batch output filenames. This keeps filenames "
+    "stable, but requires that the filename_extra variable be unique for each "
+    "test run. Results will be overwritten and a warning emitted if not.")
+
 parser.add_argument(
     "--batch-repetitions", action="store", type=int, dest="BATCH_REPS",
     metavar="REPETITIONS",
@@ -298,7 +311,7 @@ test_group = parser.add_argument_group(
 
 test_group.add_argument(
     "-H", "--host",
-    action="append", type=unicode, dest="HOSTS", metavar='HOST', default=[],
+    action=AddHost, type=unicode, dest="HOSTS", metavar='HOST', default=[],
     help="Host to connect to for tests. For tests that support it, multiple "
     "hosts can be specified by supplying this option multiple times. Hosts can "
     "also be specified as unqualified arguments; this parameter guarantees that "
@@ -356,6 +369,22 @@ test_group.add_argument(
     "detected to support this (requires netperf version 2.7 or newer). Set to 0 "
     "to disable.")
 
+test_group.add_argument(
+    "--send-size",
+    action="append", type=unicode, dest="SEND_SIZE", default=[],
+    help="Send size (in bytes) used for TCP tests. netperf uses the socket "
+    "buffer size by default, which if too large can cause spikes in the "
+    "throughput results. Lowering this value will increase CPU usage but "
+    "also improves the fidelity of the throughput results without having "
+    "to decrease the socket buffer size. Can be specified multiple times, "
+    "with each value corresponding to a stream of a test.")
+
+test_group.add_argument(
+    "--test-payload",
+    action="store", type=unicode, dest="TEST_PAYLOAD", default='/dev/urandom',
+    help="Path to file containing payload to pre-fill the netperf buffers with "
+    "defaults to randomised payload")
+
 test_group.add_argument(
     "--test-parameter",
     action=Update, type=keyval, dest="TEST_PARAMETERS", metavar='key=value',
@@ -382,6 +411,20 @@ test_group.add_argument(
     "system, and can fail if there are too many simultaneous upload flows; which "
     "is why this option is not enabled by default.")
 
+test_group.add_argument(
+    "--marking-name",
+    action=Update, dest="MARKING_NAMES", metavar='name=hexcode',
+    type=keyval_transformer(keyfunc=lambda x: x.upper(),
+                            valfunc=util.parse_int,
+                            errmsg="Values must be integers"),
+    help="Define a new symbolic name that can be used when specifying flow "
+    "markings using the 'markings' test parameter. This can be used to make "
+    "it easier to specify custom diffserv markings on flows by using symbolic "
+    "names for each marking value instead of the hex codes. Values specified "
+    "here will be used in addition to the common values (AFxx, CSx and EF - "
+    "see man page for full list), and cannot override the built-in names. "
+    "Names will be case-folded when matching.")
+
 plot_group = parser.add_argument_group(
     "Plot configuration",
     "These options are used to configure the appearance of "
@@ -423,11 +466,12 @@ tool_group.add_argument(
 
 tool_group.add_argument(
     "--http-getter-urllist",
-    action="store", type=unicode, dest="HTTP_GETTER_URLLIST", metavar="FILENAME",
+    action="append", type=unicode, dest="HTTP_GETTER_URLLIST", metavar="FILENAME",
     help="Filename containing the list of HTTP URLs to get. Can also be a URL, "
     "which will then be downloaded as part of each test iteration. If not "
     "specified, this is set to http://<hostname>/filelist.txt where <hostname> "
-    "is the first test hostname.")
+    "is the first test hostname. Can be specified multiple times, in which case "
+    "one http-getter instance will be run for each URL list.")
 
 tool_group.add_argument(
     "--http-getter-dns-servers",
@@ -448,12 +492,28 @@ tool_group.add_argument(
     default=4, help="Number of workers to use for getting HTTP urls. "
     "Default is 4.")
 
+tool_group.add_argument(
+    "--irtt-sampling-interval",
+    action="store", type=int, dest="IRTT_INTERVAL", metavar="MILLISECONDS",
+    help="Override the sampling interval passed to irtt, in milliseconds. Can be "
+    "used to run irtt with a higher sampling frequency than the rest of the test. "
+    "If set, this will override the sampling interval for all instances of irtt "
+    "used in the test.")
+
 misc_group = parser.add_argument_group("Misc and debugging options")
 
 misc_group.add_argument(
     "-L", "--log-file",
-    action=LogFile, type=unicode, dest="LOG_FILE",
-    help="Write debug log (test program output) to log file.")
+    action=LogFile, type=unicode, dest="LOG_FILE", nargs='?',
+    help="Write debug log (test program output) to log file. If the option is "
+    "enabled but no file name is given, the log file name is derived from the "
+    "test data filename.")
+
+misc_group.add_argument(
+    "--cache-file",
+    action="store", type=unicode, dest="CACHE_FILE", metavar="FILENAME",
+    help="Use this file for caching lookup of runner file names and "
+    "versions between runs.")
 
 misc_group.add_argument(
     '--list-tests',
@@ -508,7 +568,7 @@ class Settings(argparse.Namespace):
 
         if not os.path.exists(filename):
             # Test not found, assume it's a hostname
-            self.HOSTS.append(test_name)
+            append_host(self.HOSTS, test_name)
         elif self.NAME is not None and self.NAME != test_name:
             raise RuntimeError("Multiple test names specified.")
         else:
@@ -523,6 +583,7 @@ class Settings(argparse.Namespace):
                            OLD_RCFILE, self.RCFILE)
             self.RCFILE = OLD_RCFILE
         if os.path.exists(self.RCFILE):
+            logger.debug("Loading rc file %s", self.RCFILE)
 
             config = RawConfigParser()
             config.optionxform = lambda x: x.upper()
@@ -556,6 +617,7 @@ class Settings(argparse.Namespace):
                     vals[k] = False
                 else:
                     raise ValueError("Not a boolean: %s" % v)
+                logger.debug("Set value %s=%s from rc file", k, vals[k])
                 continue
 
             elif t:
@@ -572,6 +634,7 @@ class Settings(argparse.Namespace):
                     vals[k] = [t(i.strip()) for i in token_split(v)]
                 else:
                     vals[k] = val
+                logger.debug("Set value %s=%s from rc file", k, vals[k])
 
         return vals
 
@@ -698,6 +761,9 @@ class Settings(argparse.Namespace):
         elif self.NEW_GUI_INSTANCE:
             self.GUI = True
 
+        if self.CACHE_FILE:
+            global_cache.read_file(self.CACHE_FILE)
+
         if self.REMOTE_METADATA:
             self.EXTENDED_METADATA = True
 
@@ -716,10 +782,12 @@ class Settings(argparse.Namespace):
         if self.DATA_DIR is None:
             self.DATA_DIR = os.path.dirname(self.OUTPUT) or '.'
 
-        # Backwards compatibility for when LOCAL_BIND could only be specified
-        # once - just duplicate the value
+        # For per-stream options, if only specified once duplicate them for each
+        # host so they apply to all flows
         if len(self.LOCAL_BIND) == 1 and len(self.HOSTS) > 1:
             self.LOCAL_BIND *= len(self.HOSTS)
+        if len(self.SEND_SIZE) == 1 and len(self.HOSTS) > 1:
+            self.SEND_SIZE *= len(self.HOSTS)
 
         for k, v in self.BATCH_OVERRIDE.items():
             if not hasattr(v, 'lower'):
@@ -736,6 +804,9 @@ def load_gui(settings):
 
 
 def load(argv):
+    logger.info("Starting Flent %s using Python %s.", VERSION,
+                sys.version.split()[0])
+
     # We parse the args twice - the first pass is just to get the test name and
     # the name of the rcfile to parse in order to get the defaults
     settings = parser.parse_args(argv, namespace=Settings(DEFAULT_SETTINGS))
@@ -763,9 +834,6 @@ def load(argv):
     if settings.LIST_PLOTS:
         list_plots(settings)
 
-    logger.info("Started Flent %s using Python %s.", VERSION,
-                sys.version.split()[0])
-
     return settings
 
 
